"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

# -*- coding: utf-8 -*-
from flask import current_app
from constants import *
from utilities.config_manager import config_manager
from distutils.version import LooseVersion
from libs import base
import sys
import uuid


def get_cluster_id(
    cluster_details, session, session_manager, cluster_manager
):

    node_address = input_node_address = cluster_details.get('seed_node', '')
    input_node_list = cluster_details.get('node_list', {})
    if not input_node_list:
        # Check in multicluster list of this session (if present) for other
        # nodes of this cluster; Useful when this requested cluster has
        # int-ext ip mapping & mc might already contain this ip-map if
        # mentioned in existing monitored clusters' xdr configuration
        mc_obj = session_manager.get_multicluster(session.get('session', None))
        if mc_obj:
            input_node_list = mc_obj.get_cluster_ipmap(node_address)

    ip, port = node_address.split(":")
    security_enabled = base.is_security_enabled(ip, port)

    if security_enabled is False and ip == 'localhost':
        node_address = base.get_self_address(ip, port)
        ip, port = node_address.split(":")
    cluster_name = cluster_details.get('cluster_name')
    if 'cluster_name' not in cluster_details:
        cluster_name = None
    from libs import citrusleaf as cl

    session_key = session.get('session', None)

    cluster_id = None
    response = None
    #response = cluster_manager.get_cluster_info(node_address, user='', password='')
    if security_enabled:
        if session_key is not None:
            cls_id = session_manager.is_cluster_present(session_key, node_address, search_node_list=True, cls_mgr=cluster_manager)
            if cls_id is not False:
                if cluster_name is not None:
                    session_manager.update_cluster_name(session_key, cls_id, cluster_name)

                response = cluster_manager.get_cluster_data_by_cluster_id(cls_id)
                if response is not None:
                    response['status'] = 'success'
                    cls_obj = cluster_manager.get_cluster_by_id(cls_id)
                    response['build_details'] = cls_obj.get_build_no_of_cluster_nodes()
                    response['nodes_compatibility'] = cls_obj.get_cluser_compatibility_details()
                    if cls_obj.is_paused() is True:
                        cls_obj.resume()
                    return response
    else:
        response = cluster_manager.get_cluster_obj_by_node(node_address)

    if response is not None and session_key is not None:
        resp = cluster_manager.get_cluster_data_by_cluster_object(response)
        """IF CLUSTER ID IN SESSION AND CLUSTER MANAGER ARE DIFF THEN
            REPLACE SESSION VALUES"""
        clusters_in_session = session_manager.get_clusters_from_session(session_key)
        for cluster in clusters_in_session:
            if cluster['seed_node'] == resp['seed_address']:
                if str(cluster['cluster_id']) != str(resp['cluster_id']):
                    session_manager.update_cluster_id(session_key, cluster['cluster_id'], resp['cluster_id'])
                    cluster['cluster_id'] = resp['cluster_id']

        if session_manager.is_cluster_present(session_key, node_address, cluster_name) is not False:
            #resp =  cluster_manager.get_cluster_data_by_cluster_object(response)
            resp['status'] = 'success'
            if resp['seed_address'] != node_address:
                if node_address in resp['nodes']:
                    resp['seed_address'] = str(node_address)
            cls_obj = cluster_manager.get_cluster_by_id(resp['cluster_id'])

            if cluster_name is not None:
                session_manager.update_cluster_name(session_key, resp['cluster_id'], cluster_name)

            resp['build_details'] = cls_obj.get_build_no_of_cluster_nodes()
            resp['nodes_compatibility'] = cls_obj.get_cluser_compatibility_details()
            if cls_obj.is_paused() is True:
                cls_obj.resume()
            return resp
        elif node_address in resp['nodes'] and session_manager.is_cluster_id_present(session_key, resp['cluster_id']):
            """Other Node in same cluster"""
            resp['status'] = 'success'
            if resp['seed_address'] != node_address:
                if node_address in resp['nodes']:
                    resp['seed_address'] = str(node_address)
            cls_obj = cluster_manager.get_cluster_by_id(resp['cluster_id'])

            if cluster_name is not None:
                session_manager.update_cluster_name(session_key, resp['cluster_id'], cluster_name)

            resp['build_details'] = cls_obj.get_build_no_of_cluster_nodes()
            resp['nodes_compatibility'] = cls_obj.get_cluser_compatibility_details()
            if cls_obj.is_paused() is True:
                cls_obj.resume()
            return resp

    if session_manager.is_valid_session(session_key):
        cluster_id = session_manager.is_cluster_present(session_key, node_address, cluster_name)
    else:
        if security_enabled is False:
            resp = cluster_manager.get_cluster_data_by_node(node_address)
            if resp is not None:
                session_key = uuid.uuid4()
                session['session'] = session_key
                session_manager.create_session(session_key)
                cls_obj = cluster_manager.get_cluster_by_id(resp['cluster_id'])
                node_obj = cls_obj.get_node(cls_obj.get_seed())
                session_manager.add_cluster(
                    session['session'], node_address,
                    node_obj.get_access_address(), resp['cluster_id'],
                    cluster_name)
                response = cluster_manager.get_cluster_data_by_cluster_id(resp['cluster_id'])
                response['status'] = 'success'
                cls_obj = cluster_manager.get_cluster_by_id(resp['cluster_id'])
                response['build_details'] = cls_obj.get_build_no_of_cluster_nodes()
                response['nodes_compatibility'] = cls_obj.get_cluser_compatibility_details()

                if cluster_name is not None:
                    session_manager.update_cluster_name(session_key, resp['cluster_id'], cluster_name)

                if cls_obj.is_paused() is True:
                    cls_obj.resume()
                return response

    if cluster_id:
        if cluster_name is not None:
            session_manager.update_cluster_name(session_key, cluster_id, cluster_name)

        response = cluster_manager.get_cluster_data_by_cluster_id(cluster_id)
        response['status'] = 'success'
        cls_obj = cluster_manager.get_cluster_by_id(cluster_id)
        response['build_details'] = cls_obj.get_build_no_of_cluster_nodes()
        response['nodes_compatibility'] = cls_obj.get_cluser_compatibility_details()
        if cls_obj.is_paused() is True:
            cls_obj.resume()
        return response
    elif 'username' in cluster_details.keys() and 'password' in cluster_details.keys():
        response = cluster_manager.get_cluster_info(
                node_address, input_node_address, input_node_list,
                user=str(cluster_details['username']),
                password=cl.hashpassword(str(cluster_details['password'])))
    else:
        response = cluster_manager.get_cluster_info(
            node_address, input_node_address, input_node_list,
            user='', password='')

    if 'error' not in response:
        if response['cluster_id'] is not None:
            response['status'] = 'success'
            cls_obj = cluster_manager.get_cluster_by_id(response['cluster_id'])
            response['build_details'] = cls_obj.get_build_no_of_cluster_nodes()
            response['nodes_compatibility'] = cls_obj.get_cluser_compatibility_details()
            if not session_manager.is_valid_session(session_key):
                session_key = uuid.uuid4()
                session['session'] = session_key
                session_manager.create_session(session_key)
            if 'username' in cluster_details.keys():

                seed_ip, seed_port = str(response['seed_address']).split(":")

                #user_roles = base.get_cluster_userroles(seed_ip, seed_port,
                #   user=str(request.form['username']),
                #   password=str(cl.hashpassword(str(request.form['password']))))

                #if 'sys-admin' in user_roles:
                # s_ip, s_port = str(response['seed_address']).split(':')
                node_obj = cls_obj.get_node(cls_obj.get_seed())
                s_ip, s_port = node_obj.get_access_address().rsplit(":", 1)
                build = base.get_node_build(s_ip, s_port, user=str(cluster_details['username']), password=cl.hashpassword(str(cluster_details['password'])) )
                user_roles = base.get_cluster_userroles(s_ip, s_port,user=str(cluster_details['username']), password=cl.hashpassword(str(cluster_details['password'])) )
                if not user_roles:
                    return dict(status='failure', error='No roles assigned to the user')

                # build = int(str(build).split("-")[0].replace(".", ""))
                # if build > 354 or build == 354:

                if LooseVersion(build) > LooseVersion('3.5.4') or \
                        LooseVersion(build) == LooseVersion('3.5.4'):
                    for role in user_roles:
                        status, privileges = base.query_role(s_ip, s_port, str(cluster_details['username']), cl.hashpassword(str(cluster_details['password'])), role)
                        if status != 0:
                            err_msg = ERROR_DESCRIPTION[status] if status in ERROR_DESCRIPTION else 'Unknown Error..Server Returned: ' + str(status)
                            return dict(status='failure', error=err_msg)

                session_manager.add_cluster(
                    session['session'], response['seed_address'],
                    node_obj.get_access_address(), response['cluster_id'],
                    cluster_name, user=str(cluster_details['username']),
                    password=cl.hashpassword(str(cluster_details['password'])),
                    pl_password=str(cluster_details['password']))
                import time
                import datetime
                ts = time.time()
                st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H:%M:%S')
                sys.stderr.write('\n' + str(st) + ' : Logging in..')
                #else:
                 #   response['status'] = 'failure'
                  #  response['error'] = "You don't have enough permissions"

            else:
                node_obj = cls_obj.get_node(cls_obj.get_seed())
                session_manager.add_cluster(
                    session['session'], node_address,
                    node_obj.get_access_address(), response['cluster_id'],
                    cluster_name)
    else:
        response['status'] = 'failure'
    return response


def get_info(cluster_obj, user=None, password=None):

    cluster_obj.get_updates(alerts_flag=True, user=user, password=password)
    info = dict()
    # info['nodes'] = cluster_obj.get_node_list().values()
    info['nodes'] = cluster_obj.get_node_list()
    info['namespaces'] = list(cluster_obj.get_namespaces(user=user, password=password))
    info['cluster_status'] = cluster_obj.get_status()
    info['indexes'] = cluster_obj.get_index_list_by_namespace(user=user, password=password)
    info['build'] = cluster_obj.get_build_no(user=user, password=password)
    info['update_interval'] = config_manager.get_update_interval()
    info['different_cluster_nodes'] = cluster_obj.get_different_cluster_nodes()
    info['cluster_builds'] = list(cluster_obj._cluster_builds)
    return info


def get_basic_info(cluster_obj, user=None, password=None):

    cluster_obj.get_updates(alerts_flag=True, user=user, password=password)
    info = dict()
    # info['nodes'] = cluster_obj.get_node_list().values()
    info['nodes'] = cluster_obj.get_node_list()
    info['off_nodes'] = cluster_obj.get_off_nodes(user=user, password=password)
    info['memory'] = cluster_obj.get_memory_usage()
    info['disk'] = cluster_obj.get_disk_usage()
    info['namespaces'] = list(cluster_obj.get_namespaces(user=user, password=password))
    info['cluster_status'] = cluster_obj.get_status()
    info['build'] = cluster_obj.get_build_no(user=user, password=password)
    info['update_interval'] = config_manager.get_update_interval()
    info['active_red_alert_count'] = cluster_obj.get_active_red_alert_count(user=user, password=password)
    #info['dc_names'] = cluster_obj.get_dc_names(user=user, password=password)
    info['build_details'] = cluster_obj.get_build_no_of_cluster_nodes()
    info['users'] = cluster_obj._users
    info['cluster_builds'] = list(cluster_obj._cluster_builds)
    version_list = info['build_details']['version_list']
    if len(version_list) == 0 or len(version_list) == 1:
        info['nodes_compatibility'] = 'homogeneous'
    else:
        min_v = max_v = version_list.keys()[0]
        flag = True
        for key in version_list.keys():
            if LooseVersion(key) < LooseVersion(min_v):
                min_v = key
                flag = False
            elif LooseVersion(key) > LooseVersion(max_v):
                max_v = key
                flag = False
        if flag is True:
            info['nodes_compatibility'] = 'homogeneous'
        else:
            # min_v = int(str(min_v).split("-")[0].replace(".", ""))
            # max_v = int(str(max_v).split("-")[0].replace(".", ""))
            # min_v = int(str(min_v).replace(".", ""))
            # max_v = int(str(max_v).replace(".", ""))
            # if min_v < 354 and max_v >= 354:
            if LooseVersion(min_v) < LooseVersion('3.5.4') and \
                            LooseVersion(max_v) >= LooseVersion('3.5.4'):
                info['nodes_compatibility'] = 'incompatible'
            else:
                info['nodes_compatibility'] = 'compatible'
    return info


def get_clusters(clusters):

    clusters_info = dict(clusters=[{"id": cluster_id, "nodes": cluster_obj.get_node_list()}
        for cluster_id, cluster_obj in list(clusters.get_clusters().items())])
    return clusters_info


def get_throughput(cluster_obj, node_addresses, user=None, password=None):

    if node_addresses is None:
        node_addresses = cluster_obj.get_node_list()
    else:
        node_addresses = node_addresses.split(',')

    throughput = dict(read_tps={}, write_tps={}, cluster_status=cluster_obj.get_status())
    for node_address in node_addresses:
        node = cluster_obj.get_node(node_address)
        if node is None or node.get_status(user=user, password=password) == "off" or not node.is_valid():
            continue
        #throughput["read_tps"][node_address] = node.get_statistic("read_tps")
        #throughput["write_tps"][node_address] = node.get_statistic("write_tps")
        throughput["read_tps"][node_address] = node.get_read_tps()
        throughput["write_tps"][node_address] = node.get_write_tps()
    return throughput


def get_throughput_history(cluster_obj, node_addresses, start_time=None, end_time=None,
                                user=None, password=None):

    if node_addresses is None:
        node_addresses = cluster_obj.get_node_list()
    else:
        node_addresses = node_addresses.split(',')

    throughput_history = dict(read_tps={}, write_tps={}, cluster_status=cluster_obj.get_status())
    for node_address in node_addresses:
        node = cluster_obj.get_node(node_address)
        if node is None:
            continue
        throughput_history["read_tps"][node_address] = list(node.get_read_tps_history(start_time, end_time))
        throughput_history["write_tps"][node_address] = list(node.get_write_tps_history(start_time, end_time))
    return throughput_history


def get_alerts(cluster_obj, last_id, user=None, password=None):

    cluster_obj.delete_old_alerts()
    return cluster_obj.get_alerts(last_id, user=user, password=password)


def get_indexes_by_namespace(cluster_obj, namespace_name,
                                        user=None, password=None):

    result = {"cluster_status": "on"}

    indexes = cluster_obj.get_indexes_by_namespaces(namespace_name,
                                                user=user, password=password)

    if indexes == UNABLE_TO_CONNECT:
        result["cluster_status"] = "off"
        return result

    result["indexes"] = indexes
    return result


def get_sets_by_namespace(cluster_obj, namespace_name, user=None, password=None):

    result = {"cluster_status": "on"}

    sets = cluster_obj.get_sets_info_by_namespaces(namespace_name,
                                            user=user, password=password)
    if sets == UNABLE_TO_CONNECT:
        result["cluster_status"] = "off"
        return result

    result["sets"] = sets
    return result


def add_node(cluster_obj, node_address, user=None, password=None):

    return cluster_obj.add_existing_node(node_address, user=user, password=password)


def remove_node(cluster_obj, node_address):

    return cluster_obj.remove_node(node_address)


def initiate_backup(ansible_present, cluster_obj, namespace, destination_node_address,
destination_location, sets, only_metadata, scan_priority, terminate_on_change,
username="", password=None, usr=None, passwd=None, pl_password=None):

    return cluster_obj.initiate_backup(ansible_present, namespace, destination_node_address,
    username, password, destination_location, sets, only_metadata,
    scan_priority, terminate_on_change, usr=usr, passwd=passwd, pl_password=pl_password)


def get_backup_progress(ansible_present, cluster_obj):

    return cluster_obj.get_backup_progress(ansible_present)


def get_successful_backups(cluster_obj):

    return cluster_obj.get_successful_backups()


def get_available_backups(ansible_present, cluster_obj, source_node_address, source_location,
username, password):

    return cluster_obj.get_available_backups(ansible_present, source_node_address,
    source_location, username, password)


def initiate_restore(ansible_present, cluster_obj, source_node_address, source_location, threads,
namespace, missing_records_only, ignore_generation_num, username="",
password=None, usr=None, passwd=None, pl_password=None):

    return cluster_obj.initiate_restore(ansible_present, source_node_address, source_location,
    threads, namespace, missing_records_only, ignore_generation_num, username,
    password, usr=usr, passwd=passwd, pl_password=pl_password)


def get_restore_progress(
        ansible_present, cluster_obj, user=None, password=None):
    return cluster_obj.get_restore_progress(
        ansible_present, user=user, password=password)


def get_user_roles(cluster_obj, user, passwd):

    """
    Get user roles for a user on a agiven cluster
    """
    roles = cluster_obj.get_user_roles(user, passwd)
    if roles is None : return []
    return roles

def add_user(cluster_obj, user, passwd, roles):

    response = cluster_obj.add_new_user(user, passwd, roles)
    return response


def remove_user(cluster_obj, user):

    response = cluster_obj.drop_user(user)
    return response


def update_user(cluster_obj, user, roles, old_roles, passwd=None):

    response, status = cluster_obj.update_user(user, roles, old_roles, passwd)
    return response, status


def get_all_users(cluster_obj):

    response = cluster_obj.get_all_users()
    return response


def change_password(cluster_obj, user, old_passwd, new_passwd):

    response = cluster_obj.change_password(user, old_passwd, new_passwd)
    return response


def close_sockets(cluster_obj):

    cluster_obj.close_sockets()

def get_all_roles(cluster_obj):

    response = cluster_obj.get_all_roles()
    return response

def get_role(cluster_obj, role):
    
    response = cluster_obj.get_role(role)
    return response    

def add_role(cluster_obj, role, privileges):
    
    response = cluster_obj.add_role(role, privileges)
    return response

def drop_role(cluster_obj, role):

    response = cluster_obj.drop_role(role)
    return response

def add_privileges(cluster_obj, role, privileges):

    response = cluster_obj.add_privileges(role, privileges)
    return response

def drop_privileges(cluster_obj, role, privileges):

    response = cluster_obj.drop_privileges(role, privileges)
    return response

def update_role(cluster_obj, role, privileges, old_privileges):

    response = cluster_obj.update_role(role, privileges, old_privileges)
    return response

def fire_cmd(cluster_obj, cmd, user=None, password=None):

    return cluster_obj.fire_cmd(cmd, user=user, password=password)
